import { __decorate, __param, __metadata } from 'tslib';
import { flatten, isEqual, mapValues, sortBy } from 'lodash-es';
import { BehaviorSubject, combineLatest, from, of, switchMap, isObservable, map, distinctUntilChanged } from 'rxjs';
import { combineLatestOrDefault, parse, Expression, parseTemplate } from '@adminide-stack/extension-api';
import { SERVER_TYPES, ClientContainerService } from 'common';
import { injectable, inject } from 'inversify';

/** Manages and executes contributions from all extensions. */
let ContributionRegistry = class ContributionRegistry {
  viewerService;
  modelService;
  contextKeyService;
  /** All entries, including entries that are not enabled in the current context. */
  _entries = new BehaviorSubject([]);
  constructor(viewerService, modelService, contextKeyService) {
    this.viewerService = viewerService;
    this.modelService = modelService;
    this.contextKeyService = contextKeyService;
  }
  /**
   * Register contributions and return an unsubscribable that deregisters the contributions.
   * Any expressions in the contributions need to be already parsed for fast re-evaluation.
   */
  registerContributions(entryToRegister) {
    this._entries.next([...this._entries.value, entryToRegister]);
    return {
      unsubscribe: () => {
        this._entries.next(this._entries.value.filter(entry => Object.keys(entry)[0] !== Object.keys(entryToRegister)[0]));
      },
      entry: entryToRegister
    };
  }
  deregisterContributions(entryToDeregister) {
    this._entries.next(this._entries.value.filter(entry => {
      return Object.keys(entry)[0] !== Object.keys(entryToDeregister)[0];
    }));
    return true;
  }
  /**
   * Atomically deregister the previous contributions and register the next contributions. The registry's observable
   * emits only one time after both operations are complete (instead of also emitting after the deregistration
   * and before the registration).
   */
  replaceContributions(previous, next) {
    this._entries.next([...this._entries.value.filter(entry => entry !== previous.entry), next]);
    return {
      unsubscribe: () => {
        this._entries.next(this._entries.value.filter(entry => entry !== next));
      },
      entry: next
    };
  }
  /**
   * Returns an observable that emits all contributions (merged) evaluated in the current model
   * (with the optional scope). It emits whenever there is any change.
   *
   * @template T Extra allowed property value types for the {@link Context} value. See
   * {@link Context}'s `T` type parameter for more information.
   * @param scope The scope in which contributions are fetched. A scope can be a sub-component of
   * the UI that defines its own context keys, such as the hover, which stores useful loading and
   * definition/reference state in its scoped context keys.
   * @param extraContext Extra context values to use when computing the contributions. Properties
   * in this object shadow (take precedence over) properties in the global context for this
   * computation.
   */
  getContributions(scope) {
    return this.getContributionsFromEntries(this._entries, scope);
  }
  /**
   * @template T Extra allowed property value types for the {@link Context} value. See {@link Context}'s `T` type
   * parameter for more information.
   */
  getContributionsFromEntries(entries, scope, logWarning = (...args) => console.log(...args)) {
    return combineLatest([entries.pipe(switchMap(entries => combineLatestOrDefault(entries.length > 0 ? entries.map(entry => {
      const contribution = Object.values(entry)[0];
      return isObservable(contribution) ? contribution : of(contribution);
    }) : [of({})]))), from(this.viewerService.activeViewerUpdates).pipe(map(activeEditor => {
      if (!activeEditor) {
        return undefined;
      }
      switch (activeEditor.type) {
        case 'CodeEditor':
          return {
            ...activeEditor,
            model: this.modelService.getPartialModel(activeEditor.resource)
          };
        case 'DirectoryViewer':
          return activeEditor;
      }
    })), of(this.contextKeyService.getContext({}))
    // this.settings @sri commented, since it is not used.
    ]).pipe(map(([multiContributions, activeEditor, context]) => {
      // Merge in extra context.
      // if (extraContext) {
      //     context = { ...context, ...extraContext }
      // }
      // TODO(sqs): Observe context so that we update immediately upon changes.
      // const computedContext = computeContext(activeEditor, settings, context, scope)
      // using lodash flatten instead of esnext flat due to test failures
      return flatten(multiContributions).map(contributions => {
        try {
          return filterContributions(evaluateContributions(context.getValue(''), contributions));
        } catch (error) {
          // An error during evaluation causes all of the contributions in the same entry to be
          // discarded.
          logWarning('Discarding contributions: evaluating expressions or templates failed.', {
            contributions,
            error
          });
          return {};
        }
      });
    }), map(mergeContributions), distinctUntilChanged((a, b) => isEqual(a, b)));
  }
  /**
   * All contribution entries, emitted whenever the set of registered contributions changes.
   *
   * Most callers should use ContributionsRegistry#getContributions. Only use #entries if the
   * caller needs information that is discarded when the contributions are merged (such as the
   * extension that registered each set of contributions).
   */
  entries = this._entries;
};
ContributionRegistry = __decorate([injectable(), __param(0, inject(SERVER_TYPES.IViewerService)), __param(1, inject(SERVER_TYPES.IModelService)), __param(2, inject(ClientContainerService.IContextKeyService)), __metadata("design:paramtypes", [Object, Object, Object])], ContributionRegistry);
/**
 * Merges the contributions.
 *
 * Most callers should use ContributionRegistry#getContributions, which merges all registered
 * contributions.
 */
function mergeContributions(contributions) {
  if (contributions.length === 0) {
    return {};
  }
  if (contributions.length === 1) {
    return contributions[0];
  }
  const merged = {};
  for (const contribution of contributions) {
    // swallow errors from malformed manifests to prevent breaking other
    // contributions or extensions: https://github.com/sourcegraph/sourcegraph/pull/12573
    if (contribution.actions) {
      try {
        if (!merged.actions) {
          merged.actions = [...contribution.actions];
        } else {
          merged.actions = [...merged.actions, ...contribution.actions];
        }
      } catch {
        // noop
      }
    }
    if (contribution.menus) {
      try {
        if (!merged.menus) {
          merged.menus = {
            ...contribution.menus
          };
        } else {
          for (const [menu, items] of Object.entries(contribution.menus)) {
            const mergedItems = merged.menus[menu];
            try {
              if (!mergedItems) {
                merged.menus[menu] = [...items];
              } else {
                merged.menus[menu] = [...mergedItems, ...items];
              }
            } catch {
              // noop
            }
          }
        }
      } catch {
        // noop
      }
    }
    if (contribution.views) {
      try {
        if (!merged.views) {
          merged.views = [...contribution.views];
        } else {
          merged.views = [...merged.views, ...contribution.views];
        }
      } catch {
        // noop
      }
    }
    if (contribution.searchFilters) {
      try {
        if (!merged.searchFilters) {
          merged.searchFilters = [...contribution.searchFilters];
        } else {
          merged.searchFilters = [...merged.searchFilters, ...contribution.searchFilters];
        }
      } catch {
        // noop
      }
    }
    if (contribution.tables) {
      try {
        if (!merged.tables) {
          merged.tables = {
            ...contribution.tables
          };
        } else {
          for (const [table, columns] of Object.entries(contribution.tables)) {
            const mergedItems = merged.tables[table];
            const newColumns = columns?.map((column, index) => ({
              ...column,
              position: column.position || index + 1
            }));
            try {
              if (!mergedItems) {
                merged.tables[table] = sortBy([...newColumns], column => column.position);
              } else {
                merged.tables[table] = sortBy([...mergedItems, ...newColumns], column => column.position);
              }
            } catch {
              // noop
            }
          }
        }
      } catch {
        // noop
      }
    }
    if (contribution.buttons) {
      try {
        if (!merged.buttons) {
          merged.buttons = {
            ...contribution.buttons
          };
        } else {
          for (const [button, items] of Object.entries(contribution.buttons)) {
            const mergedItems = merged.buttons[button];
            try {
              if (!mergedItems) {
                merged.buttons[button] = [...items];
              } else {
                merged.buttons[button] = [...mergedItems, ...items];
              }
            } catch {
              // noop
            }
          }
        }
      } catch {
        // noop
      }
    }
    if (contribution.pageNavBar) {
      try {
        if (!merged.pageNavBar) {
          merged.pageNavBar = [...contribution.pageNavBar];
        } else {
          merged.pageNavBar = [...merged.pageNavBar, ...contribution.pageNavBar];
        }
        merged.pageNavBar = sortBy(merged.pageNavBar, item => item.priority);
      } catch {
        // noop
      }
    }
    if (contribution.configValue) {
      try {
        if (!merged.configValue) {
          merged.configValue = {
            ...contribution.configValue
          };
        } else {
          merged.configValue = {
            ...merged.configValue,
            ...contribution.configValue
          };
        }
      } catch {
        // noop
      }
    }
  }
  return merged;
}
/**
 * Filters the contributions to only those that are enabled in the current context.
 */
function filterContributions(contributions) {
  if (!contributions.menus) {
    return contributions;
  }
  return {
    ...contributions,
    menus: mapValues(contributions.menus, menuItems => menuItems?.filter(menuItem => menuItem.when !== false))
  };
}
/**
 * Evaluates expressions in contribution definitions against the given context.
 *
 * @todo could walk object recursively
 */
function evaluateContributions(context, contributions) {
  return {
    ...contributions,
    menus: contributions.menus && mapValues(contributions.menus, menuItems => {
      return menuItems?.map(menuItem => {
        const when = typeof menuItem.when === 'string' ? parse(menuItem.when) : undefined;
        return {
          ...menuItem,
          when: when && !!when.exec(context)
        };
      });
    }),
    tables: contributions.tables && mapValues(contributions.tables, columns => columns?.map(column => ({
      ...column,
      when: column.when && !!column.when.exec(context)
    }))),
    pageNavBar: contributions.pageNavBar && contributions.pageNavBar.map(item => ({
      ...item,
      name: item.name
    })),
    actions: evaluateActionContributions(context, contributions.actions)
  };
}
/**
 * Evaluates expressions in contribution definitions against the given context.
 */
function evaluateActionContributions(context, actions) {
  return actions?.map(action => ({
    ...action,
    title: action.title?.exec(context),
    category: action.category?.exec(context),
    description: action.description?.exec(context),
    iconURL: action.iconURL?.exec(context),
    actionItem: action.actionItem && {
      ...action.actionItem,
      label: action.actionItem.label?.exec(context),
      description: action.actionItem.description?.exec(context),
      iconURL: action.actionItem.iconURL?.exec(context),
      iconDescription: action.actionItem.iconDescription?.exec(context),
      pressed: action.actionItem.pressed?.exec(context)
    },
    commandArguments: action.commandArguments?.map(argument => argument instanceof Expression ? argument.exec(context) : argument)
  }));
}
/**
 * Evaluates expressions in contribution definitions against the given context.
 */
function parseContributionExpressions(contributions) {
  return {
    ...contributions,
    menus: contributions.menus && mapValues(contributions.menus, menuItems => menuItems?.map(menuItem => ({
      ...menuItem,
      when: typeof menuItem.when === 'string' ? parse(menuItem.when) : undefined
    }))),
    tables: contributions.tables && mapValues(contributions.tables, columns => columns?.map(column => ({
      ...columns,
      title: column.title,
      accessor: column.accessor,
      when: typeof column.when === 'string' ? parse(column.when) : undefined
    }))),
    buttons: contributions.buttons && mapValues(contributions.buttons, buttons => buttons?.map(button => ({
      ...buttons,
      type: button.type
    }))),
    pageNavBar: contributions && contributions.pageNavBar.map(item => ({
      ...item,
      name: item.name
    })),
    actions: contributions && parseActionContributionExpressions(contributions.actions)
  };
}
const maybe = (value, function_) => value === undefined ? undefined : function_(value);
/**
 * Evaluates expressions in contribution definitions against the given context.
 */
function parseActionContributionExpressions(actions) {
  return actions?.map(action => ({
    ...action,
    title: maybe(action.title, parseTemplate),
    category: maybe(action.category, parseTemplate),
    description: maybe(action.description, parseTemplate),
    iconURL: maybe(action.iconURL, parseTemplate),
    actionItem: action.actionItem && {
      ...action.actionItem,
      label: maybe(action.actionItem.label, parseTemplate),
      description: maybe(action.actionItem.description, parseTemplate),
      iconURL: maybe(action.actionItem.iconURL, parseTemplate),
      iconDescription: maybe(action.actionItem.iconDescription, parseTemplate),
      pressed: maybe(action.actionItem.pressed, pressed => parse(pressed))
    },
    commandArguments: action.commandArguments?.map(argument => typeof argument === 'string' ? parseTemplate(argument) : argument)
  }));
}

export { ContributionRegistry, evaluateContributions, filterContributions, mergeContributions, parseContributionExpressions };
//# sourceMappingURL=contribution-service.js.map
