import { Subscription, map, switchMap, catchError } from 'rxjs';
import { combineLatestOrDefault } from '@adminide-stack/extension-api';
import { FeatureProviderRegistry } from './registry.js';
import { isDefined } from '../util/types.js';

/** Provides panel views from all extensions. */
class PanelViewProviderRegistry extends FeatureProviderRegistry {
  /**
   * Returns an observable that emits the specified panel view whenever it or the set of
   * registered panel view providers changes. If the provider emits an error, the returned
   * observable also emits an error (and completes).
   */
  getPanelView(id) {
    return getPanelView(this.entries, id);
  }
  /**
   * Returns an observable that emits all panel views whenever the set of registered panel view
   * providers or their properties change. If any provider emits an error, the error is logged and
   * the provider is omitted from the emission of the observable (the observable does not emit the
   * error).
   */
  getPanelViews(container) {
    return getPanelViews(this.entries, container);
  }
  /** @sri custom change: deregister panel view provider */
  deregisterProvider(deregistrationOptions) {
    this.entries.next(this.entries.value.filter(entry => {
      const {
        registrationOptions: option
      } = entry;
      return option.id !== deregistrationOptions.id || option.container !== deregistrationOptions.container;
    }));
    return new Subscription(() => {
      this.entries.next([...this.entries.value]);
    });
  }
}
/**
 * Exported for testing only. Most callers should use {@link ViewProviderRegistry#getPanelView},
 * which uses the registered panel view providers.
 *
 * @internal
 */
function getPanelView(entries, id) {
  return entries.pipe(map(entries => entries.find(entry => entry.registrationOptions.id === id)), switchMap(entry => entry ? addRegistrationOptions(entry) : [null]));
}
/**
 * Exported for testing only. Most callers should use {@link ViewProviderRegistry#getPanelViews},
 * which uses the registered panel view providers.
 *
 * @internal
 */
function getPanelViews(entries, container, logErrors = true) {
  return entries.pipe(switchMap(entries => combineLatestOrDefault(entries.filter(entry => entry.registrationOptions.container === container).map(entry => addRegistrationOptions(entry).pipe(catchError(error => {
    if (logErrors) {
      console.error(error);
    }
    return [null];
  })))).pipe(map(entries => entries.filter(isDefined)))));
}
function addRegistrationOptions(entry) {
  return entry.provider.pipe(map(view => view && {
    ...view,
    ...entry.registrationOptions
  }));
}

export { PanelViewProviderRegistry, getPanelView, getPanelViews };
//# sourceMappingURL=panel-views.js.map
