import { __decorate, __param, __metadata } from 'tslib';
import { Subject, BehaviorSubject, filter, takeWhile, map, startWith, throwError } from 'rxjs';
import { ClientTypes } from '@adminide-stack/extension-api';
import { ReferenceCounter } from './utils/reference-counter.js';
import { injectable, inject } from 'inversify';

const VIEWER_NOT_FOUND_ERROR_NAME = 'ViewerNotFoundError';
class ViewerNotFoundError extends Error {
  name = VIEWER_NOT_FOUND_ERROR_NAME;
  constructor(viewerId) {
    super(`Viewer not found: ${viewerId}`);
  }
}
/**
 * Creates a {@link IViewerService} instance.
 *
 * @sri we have converted to class.
 */
let ViewerService = class ViewerService {
  modelSerivce;
  id = 0;
  nextId = () => `viewer#${this.id++}`;
  /** A map of viewer ids to code viewers. */
  _viewers = new Map();
  _viewerUpdates = new Subject();
  _activeViewerUpdates = new BehaviorSubject(undefined);
  modelReferences;
  constructor(modelSerivce) {
    this.modelSerivce = modelSerivce;
    this.modelReferences = new ReferenceCounter();
  }
  /**
   * Returns the Viewer with teh given viewerId.
   * Throws if no viewer exists with the given viewerId.
   */
  getViewer = viewerId => {
    const viewer = this._viewers.get(viewerId);
    if (!viewer) {
      throw new ViewerNotFoundError(viewerId);
    }
    return viewer;
  };
  get viewers() {
    return this._viewers;
  }
  get viewerUpdates() {
    return this._viewerUpdates;
  }
  get activeViewerUpdates() {
    return this._activeViewerUpdates;
  }
  addViewer = viewerData => {
    const viewerId = this.nextId();
    if (viewerData.type === 'CodeEditor') {
      this.modelReferences.increment(viewerData.resource);
    }
    const viewer = {
      ...viewerData,
      viewerId
    };
    this._viewers.set(viewerId, viewer);
    this._viewerUpdates.next([{
      type: 'added',
      viewerId,
      viewerData
    }]);
    if (viewerData.isActive) {
      this._activeViewerUpdates.next(viewer);
    }
    return viewer;
  };
  observeViewer = ({
    viewerId
  }) => {
    try {
      const viewer = this.getViewer(viewerId);
      return this._viewerUpdates.pipe(filter(updates => updates.some(update => update.viewerId === viewerId)), takeWhile(updates => updates.every(update => update.viewerId !== viewerId || update.type !== 'deleted')), map(() => this.getViewer(viewerId)), startWith(viewer));
    } catch (error) {
      return throwError(error);
    }
  };
  setSelections({
    viewerId
  }, selections) {
    const viewer = this.getViewer(viewerId);
    if (viewer.type !== 'CodeEditor') {
      throw new Error(`Editor ID ${viewerId} is type ${String(viewer.type)}, expected CodeExitor`);
    }
    this._viewers.set(viewerId, {
      ...viewer,
      selections
    });
    this._viewerUpdates.next([{
      type: 'updated',
      viewerId,
      viewerData: {
        selections
      }
    }]);
  }
  removeViewer({
    viewerId
  }) {
    const viewer = this.getViewer(viewerId);
    this._viewers.delete(viewerId);
    this._viewerUpdates.next([{
      type: 'deleted',
      viewerId
    }]);
    // Check if this was the active viewer
    if (this._activeViewerUpdates.value && this._activeViewerUpdates.value.viewerId === viewerId) {
      this._activeViewerUpdates.next(undefined);
    }
    if (viewer.type === 'CodeEditor' && this.modelReferences.decrement(viewer.resource)) {
      this.modelSerivce.removeModel(viewer.resource);
    }
  }
  removeAllViewers() {
    const updates = [...this._viewers.keys()].map(viewerId => ({
      type: 'deleted',
      viewerId
    }));
    this._viewers.clear();
    this._viewerUpdates.next(updates);
    // The active viewer will have been removed
    this._activeViewerUpdates.next(undefined);
  }
};
ViewerService = __decorate([injectable(), __param(0, inject(ClientTypes.IModelService)), __metadata("design:paramtypes", [Object])], ViewerService);

export { ViewerService };
//# sourceMappingURL=viewer-service.js.map
